@@ -11,21 +11,27 @@ use Log::Log4perl qw(:easy);
 use Number::Bytes::Human qw( format_bytes parse_bytes );
 
 # ABSTRACT: Yet Another RESTful-Archive Service
-our $VERSION = '0.84'; # VERSION
+our $VERSION = '0.85'; # VERSION
 
 
 has secret => rand;
 
 
+sub _mojo4_compat {
+    my $self = shift;
+    $self->hook(before_dispatch => sub {
+        my($c) = @_;
+        my $stream = Mojo::IOLoop->stream($c->tx->connection);
+        return unless defined $stream;
+        $stream->timeout(3000);
+    });
+}
+
 sub startup {
     my $self = shift;
+
     if ($Mojolicious::VERSION >= 4.0) {
-        $self->hook(before_dispatch => sub {
-            my($c) = @_;
-            my $stream = Mojo::IOLoop->stream($c->tx->connection);
-            return unless defined $stream;
-            $stream->timeout(3000);
-        });
+        $self->_mojo4_compat;
     #} elsif ($Mojolicious::VERSION >= 2.37) {
     } else {
         eval { Mojo::IOLoop::Stream->timeout(3000) };
@@ -33,17 +39,14 @@ sub startup {
             WARN "error trying to set timeout: $@";
             WARN "Mojolicious version $Mojolicious::VERSION";
             WARN "Will try Mojo 4.x mode";
-            $self->hook(before_dispatch => sub {
-                my($c) = @_;
-                my $stream = Mojo::IOLoop->stream($c->tx->connection);
-                return unless defined $stream;
-                $stream->timeout(3000);
-            });
+            $self->_mojo4_compat;
         }
     }
 
     my $max_size = 53687091200;
 
+    my $tools;
+
     $self->hook(
         after_build_tx => sub {
             my ( $tx, $app ) = @_;
@@ -52,7 +55,7 @@ sub startup {
                     my $content = shift;
                     my $md5_b64 = $content->headers->header('Content-MD5') or return;
                     my $md5 = unpack 'H*', b($md5_b64)->b64_decode;
-                    my $disk = Yars::Tools->disk_for($md5) or return;
+                    my $disk = $tools->disk_for($md5) or return;
                     my $tmpdir = join '/', $disk, 'tmp';
                     -d $tmpdir or do { mkpath $tmpdir;  chmod 0777, $tmpdir; };
                     -w $tmpdir or chmod 0777, $tmpdir;
@@ -68,6 +71,16 @@ sub startup {
     );
     
     $self->SUPER::startup(@_);
+    
+    $tools = Yars::Tools->new($self->config);
+
+    $self->hook(
+        before_dispatch => sub {
+            $tools->refresh_config($self->config);
+        }
+    );
+
+    $self->helper( tools => sub { $tools } );
 
     if(my $time = $self->config->{test_expiration}) {
         require Clustericious::Command::stop;
@@ -95,7 +108,7 @@ Yars - Yet Another RESTful-Archive Service
 
 =head1 VERSION
 
-version 0.84
+version 0.85
 
 =head1 DESCRIPTION
 
